@@ -1,5 +1,4 @@
 #define _XOPEN_SOURCE 600
-#include <GLES2/gl2.h>
 #include <math.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -9,19 +8,22 @@
 #include <wayland-server-core.h>
 #include <wlr/backend.h>
 #include <wlr/backend/session.h>
+#include <wlr/render/allocator.h>
 #include <wlr/render/wlr_renderer.h>
-#include <wlr/types/wlr_box.h>
 #include <wlr/types/wlr_matrix.h>
 #include <wlr/types/wlr_output.h>
 #include <wlr/types/wlr_input_device.h>
+#include <wlr/types/wlr_keyboard.h>
 #include <wlr/types/wlr_tablet_pad.h>
 #include <wlr/types/wlr_tablet_tool.h>
+#include <wlr/util/box.h>
 #include <wlr/util/log.h>
 #include <xkbcommon/xkbcommon.h>
 
 struct sample_state {
 	struct wl_display *display;
 	struct wlr_renderer *renderer;
+	struct wlr_allocator *allocator;
 	bool proximity, tap, button;
 	double distance;
 	double pressure;
@@ -41,7 +43,7 @@ struct sample_state {
 
 struct tablet_tool_state {
 	struct sample_state *sample;
-	struct wlr_input_device *device;
+	struct wlr_tablet *wlr_tablet;
 	struct wl_listener destroy;
 	struct wl_listener axis;
 	struct wl_listener proximity;
@@ -53,7 +55,7 @@ struct tablet_tool_state {
 
 struct tablet_pad_state {
 	struct sample_state *sample;
-	struct wlr_input_device *device;
+	struct wlr_tablet_pad *wlr_tablet_pad;
 	struct wl_listener destroy;
 	struct wl_listener button;
 	struct wl_listener ring;
@@ -70,7 +72,7 @@ struct sample_output {
 
 struct sample_keyboard {
 	struct sample_state *sample;
-	struct wlr_input_device *device;
+	struct wlr_keyboard *wlr_keyboard;
 	struct wl_listener key;
 	struct wl_listener destroy;
 };
@@ -85,9 +87,14 @@ static void output_frame_notify(struct wl_listener *listener, void *data) {
 	int32_t width, height;
 	wlr_output_effective_resolution(wlr_output, &width, &height);
 
-	wlr_output_attach_render(wlr_output, NULL);
-	wlr_renderer_begin(sample->renderer, wlr_output->width, wlr_output->height);
-	wlr_renderer_clear(sample->renderer, (float[]){0.25f, 0.25f, 0.25f, 1});
+	struct wlr_output_state output_state;
+	wlr_output_state_init(&output_state);
+	struct wlr_render_pass *pass = wlr_output_begin_render_pass(wlr_output, &output_state, NULL, NULL);
+
+	wlr_render_pass_add_rect(pass, &(struct wlr_render_rect_options){
+		.box = { .width = wlr_output->width, .height = wlr_output->height },
+		.color = { 0.25, 0.25, 0.25, 1 },
+	});
 
 	float distance = 0.8f * (1 - sample->distance);
 	float tool_color[4] = { distance, distance, distance, 1 };
@@ -100,12 +107,20 @@ static void output_frame_notify(struct wl_listener *listener, void *data) {
 	float pad_height = sample->height_mm * scale;
 	float left = width / 2.0f - pad_width / 2.0f;
 	float top = height / 2.0f - pad_height / 2.0f;
-	const struct wlr_box box = {
-		.x = left, .y = top,
-		.width = pad_width, .height = pad_height,
-	};
-	wlr_render_rect(sample->renderer, &box, sample->pad_color,
-		wlr_output->transform_matrix);
+	wlr_render_pass_add_rect(pass, &(struct wlr_render_rect_options){
+		.box = {
+			.x = left,
+			.y = top,
+			.width = pad_width,
+			.height = pad_height,
+		},
+		.color = {
+			sample->pad_color[0],
+			sample->pad_color[1],
+			sample->pad_color[2],
+			sample->pad_color[3],
+		},
+	});
 
 	if (sample->proximity) {
 		struct wlr_box box = {
@@ -114,27 +129,42 @@ static void output_frame_notify(struct wl_listener *listener, void *data) {
 			.width = 16 * (sample->pressure + 1),
 			.height = 16 * (sample->pressure + 1),
 		};
-		float matrix[9];
-		wlr_matrix_project_box(matrix, &box, WL_OUTPUT_TRANSFORM_NORMAL,
-			sample->ring, wlr_output->transform_matrix);
-		wlr_render_quad_with_matrix(sample->renderer, tool_color, matrix);
+
+		// TODO: use sample->ring
+		wlr_render_pass_add_rect(pass, &(struct wlr_render_rect_options){
+			.box = box,
+			.color = {
+				tool_color[0],
+				tool_color[1],
+				tool_color[2],
+				tool_color[3],
+			},
+		});
 
 		box.x += sample->x_tilt;
 		box.y += sample->y_tilt;
 		box.width /= 2;
 		box.height /= 2;
-		wlr_render_rect(sample->renderer, &box, tool_color,
-			wlr_output->transform_matrix);
+		wlr_render_pass_add_rect(pass, &(struct wlr_render_rect_options){
+			.box = box,
+			.color = {
+				tool_color[0],
+				tool_color[1],
+				tool_color[2],
+				tool_color[3],
+			},
+		});
 	}
 
-	wlr_renderer_end(sample->renderer);
-	wlr_output_commit(wlr_output);
+	wlr_render_pass_submit(pass);
+	wlr_output_commit_state(wlr_output, &output_state);
+	wlr_output_state_finish(&output_state);
 	sample->last_frame = now;
 }
 
 static void tablet_tool_axis_notify(struct wl_listener *listener, void *data) {
 	struct tablet_tool_state *tstate = wl_container_of(listener, tstate, axis);
-	struct wlr_event_tablet_tool_axis *event = data;
+	struct wlr_tablet_tool_axis_event *event = data;
 	struct sample_state *sample = tstate->sample;
 	if ((event->updated_axes & WLR_TABLET_TOOL_AXIS_X)) {
 		sample->x = event->x;
@@ -158,14 +188,14 @@ static void tablet_tool_axis_notify(struct wl_listener *listener, void *data) {
 
 static void tablet_tool_proximity_notify(struct wl_listener *listener, void *data) {
 	struct tablet_tool_state *tstate = wl_container_of(listener, tstate, proximity);
-	struct wlr_event_tablet_tool_proximity *event = data;
+	struct wlr_tablet_tool_proximity_event *event = data;
 	struct sample_state *sample = tstate->sample;
 	sample->proximity = event->state == WLR_TABLET_TOOL_PROXIMITY_IN;
 }
 
 static void tablet_tool_button_notify(struct wl_listener *listener, void *data) {
 	struct tablet_tool_state *tstate = wl_container_of(listener, tstate, button);
-	struct wlr_event_tablet_tool_button *event = data;
+	struct wlr_tablet_tool_button_event *event = data;
 	struct sample_state *sample = tstate->sample;
 	if (event->state == WLR_BUTTON_RELEASED) {
 		sample->button = false;
@@ -183,7 +213,7 @@ static void tablet_tool_button_notify(struct wl_listener *listener, void *data)
 
 static void tablet_pad_button_notify(struct wl_listener *listener, void *data) {
 	struct tablet_pad_state *pstate = wl_container_of(listener, pstate, button);
-	struct wlr_event_tablet_pad_button *event = data;
+	struct wlr_tablet_pad_button_event *event = data;
 	struct sample_state *sample = pstate->sample;
 	float default_color[4] = { 0.5, 0.5, 0.5, 1.0 };
 	if (event->state == WLR_BUTTON_RELEASED) {
@@ -201,7 +231,7 @@ static void tablet_pad_button_notify(struct wl_listener *listener, void *data) {
 
 static void tablet_pad_ring_notify(struct wl_listener *listener, void *data) {
 	struct tablet_pad_state *pstate = wl_container_of(listener, pstate, ring);
-	struct wlr_event_tablet_pad_ring *event = data;
+	struct wlr_tablet_pad_ring_event *event = data;
 	struct sample_state *sample = pstate->sample;
 	if (event->position != -1) {
 		sample->ring = -(event->position * (M_PI / 180.0));
@@ -237,11 +267,10 @@ static void output_remove_notify(struct wl_listener *listener, void *data) {
 static void new_output_notify(struct wl_listener *listener, void *data) {
 	struct wlr_output *output = data;
 	struct sample_state *sample = wl_container_of(listener, sample, new_output);
-	struct sample_output *sample_output = calloc(1, sizeof(struct sample_output));
-	if (!wl_list_empty(&output->modes)) {
-		struct wlr_output_mode *mode = wl_container_of(output->modes.prev, mode, link);
-		wlr_output_set_mode(output, mode);
-	}
+
+	wlr_output_init_render(output, sample->allocator, sample->renderer);
+
+	struct sample_output *sample_output = calloc(1, sizeof(*sample_output));
 	sample_output->output = output;
 	sample_output->sample = sample;
 	wl_signal_add(&output->events.frame, &sample_output->frame);
@@ -249,16 +278,24 @@ static void new_output_notify(struct wl_listener *listener, void *data) {
 	wl_signal_add(&output->events.destroy, &sample_output->destroy);
 	sample_output->destroy.notify = output_remove_notify;
 
-	wlr_output_commit(output);
+	struct wlr_output_state state;
+	wlr_output_state_init(&state);
+	wlr_output_state_set_enabled(&state, true);
+	struct wlr_output_mode *mode = wlr_output_preferred_mode(output);
+	if (mode != NULL) {
+		wlr_output_state_set_mode(&state, mode);
+	}
+	wlr_output_commit_state(output, &state);
+	wlr_output_state_finish(&state);
 }
 
 static void keyboard_key_notify(struct wl_listener *listener, void *data) {
 	struct sample_keyboard *keyboard = wl_container_of(listener, keyboard, key);
 	struct sample_state *sample = keyboard->sample;
-	struct wlr_event_keyboard_key *event = data;
+	struct wlr_keyboard_key_event *event = data;
 	uint32_t keycode = event->keycode + 8;
 	const xkb_keysym_t *syms;
-	int nsyms = xkb_state_key_get_syms(keyboard->device->keyboard->xkb_state,
+	int nsyms = xkb_state_key_get_syms(keyboard->wlr_keyboard->xkb_state,
 			keycode, &syms);
 	for (int i = 0; i < nsyms; i++) {
 		xkb_keysym_t sym = syms[i];
@@ -280,63 +317,58 @@ static void new_input_notify(struct wl_listener *listener, void *data) {
 	struct sample_state *sample = wl_container_of(listener, sample, new_input);
 	switch (device->type) {
 	case WLR_INPUT_DEVICE_KEYBOARD:;
-		struct sample_keyboard *keyboard = calloc(1, sizeof(struct sample_keyboard));
-		keyboard->device = device;
+		struct sample_keyboard *keyboard = calloc(1, sizeof(*keyboard));
+		keyboard->wlr_keyboard = wlr_keyboard_from_input_device(device);
 		keyboard->sample = sample;
 		wl_signal_add(&device->events.destroy, &keyboard->destroy);
 		keyboard->destroy.notify = keyboard_destroy_notify;
-		wl_signal_add(&device->keyboard->events.key, &keyboard->key);
+		wl_signal_add(&keyboard->wlr_keyboard->events.key, &keyboard->key);
 		keyboard->key.notify = keyboard_key_notify;
-		struct xkb_rule_names rules = { 0 };
-		rules.rules = getenv("XKB_DEFAULT_RULES");
-		rules.model = getenv("XKB_DEFAULT_MODEL");
-		rules.layout = getenv("XKB_DEFAULT_LAYOUT");
-		rules.variant = getenv("XKB_DEFAULT_VARIANT");
-		rules.options = getenv("XKB_DEFAULT_OPTIONS");
 		struct xkb_context *context = xkb_context_new(XKB_CONTEXT_NO_FLAGS);
 		if (!context) {
 			wlr_log(WLR_ERROR, "Failed to create XKB context");
 			exit(1);
 		}
-		struct xkb_keymap *keymap = xkb_map_new_from_names(context, &rules,
+		struct xkb_keymap *keymap = xkb_keymap_new_from_names(context, NULL,
 			XKB_KEYMAP_COMPILE_NO_FLAGS);
 		if (!keymap) {
 			wlr_log(WLR_ERROR, "Failed to create XKB keymap");
 			exit(1);
 		}
-		wlr_keyboard_set_keymap(device->keyboard, keymap);
+		wlr_keyboard_set_keymap(keyboard->wlr_keyboard, keymap);
 		xkb_keymap_unref(keymap);
 		xkb_context_unref(context);
 		break;
 	case WLR_INPUT_DEVICE_TABLET_PAD:;
-		struct tablet_pad_state *pstate = calloc(sizeof(struct tablet_pad_state), 1);
-		pstate->device = device;
+		struct tablet_pad_state *pstate = calloc(1, sizeof(*pstate));
+		pstate->wlr_tablet_pad = wlr_tablet_pad_from_input_device(device);
 		pstate->sample = sample;
 		pstate->destroy.notify = tablet_pad_destroy_notify;
 		wl_signal_add(&device->events.destroy, &pstate->destroy);
 		pstate->button.notify = tablet_pad_button_notify;
-		wl_signal_add(&device->tablet_pad->events.button, &pstate->button);
+		wl_signal_add(&pstate->wlr_tablet_pad->events.button, &pstate->button);
 		pstate->ring.notify = tablet_pad_ring_notify;
-		wl_signal_add(&device->tablet_pad->events.ring, &pstate->ring);
+		wl_signal_add(&pstate->wlr_tablet_pad->events.ring, &pstate->ring);
 		wl_list_insert(&sample->tablet_pads, &pstate->link);
 		break;
-	case WLR_INPUT_DEVICE_TABLET_TOOL:
-		sample->width_mm = device->width_mm == 0 ?
-			20 : device->width_mm;
-		sample->height_mm = device->height_mm == 0 ?
-			10 : device->height_mm;
+	case WLR_INPUT_DEVICE_TABLET_TOOL:;
+		struct wlr_tablet *tablet = wlr_tablet_from_input_device(device);
+		sample->width_mm = tablet->width_mm == 0 ?
+			20 : tablet->width_mm;
+		sample->height_mm = tablet->height_mm == 0 ?
+			10 : tablet->height_mm;
 
-		struct tablet_tool_state *tstate = calloc(sizeof(struct tablet_tool_state), 1);
-		tstate->device = device;
+		struct tablet_tool_state *tstate = calloc(1, sizeof(*tstate));
+		tstate->wlr_tablet = tablet;
 		tstate->sample = sample;
 		tstate->destroy.notify = tablet_tool_destroy_notify;
 		wl_signal_add(&device->events.destroy, &tstate->destroy);
 		tstate->axis.notify = tablet_tool_axis_notify;
-		wl_signal_add(&device->tablet->events.axis, &tstate->axis);
+		wl_signal_add(&tablet->events.axis, &tstate->axis);
 		tstate->proximity.notify = tablet_tool_proximity_notify;
-		wl_signal_add(&device->tablet->events.proximity, &tstate->proximity);
+		wl_signal_add(&tablet->events.proximity, &tstate->proximity);
 		tstate->button.notify = tablet_tool_button_notify;
-		wl_signal_add(&device->tablet->events.button, &tstate->button);
+		wl_signal_add(&tablet->events.button, &tstate->button);
 		wl_list_insert(&sample->tablet_tools, &tstate->link);
 		break;
 	default:
@@ -344,7 +376,6 @@ static void new_input_notify(struct wl_listener *listener, void *data) {
 	}
 }
 
-
 int main(int argc, char *argv[]) {
 	wlr_log_init(WLR_DEBUG, NULL);
 	struct wl_display *display = wl_display_create();
@@ -366,11 +397,14 @@ int main(int argc, char *argv[]) {
 	state.new_input.notify = new_input_notify;
 	clock_gettime(CLOCK_MONOTONIC, &state.last_frame);
 
-	state.renderer = wlr_backend_get_renderer(wlr);
+	state.renderer = wlr_renderer_autocreate(wlr);
 	if (!state.renderer) {
 		wlr_log(WLR_ERROR, "Could not start compositor, OOM");
 		exit(EXIT_FAILURE);
 	}
+
+	state.allocator = wlr_allocator_autocreate(wlr, state.renderer);
+
 	if (!wlr_backend_start(wlr)) {
 		wlr_log(WLR_ERROR, "Failed to start backend");
 		wlr_backend_destroy(wlr);
